<?php
/**
 * Amazon Parser
 *
 * Parses data from Amazon.de
 *
 * @package Engines
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @link    http://forums.prospero.com/am-assocdevxml Amazon
 * @version $Id: amazonxml.php,v 1.23 2005/09/24 13:36:58 andig2 Exp $
 */

require_once './core/genres.php';
require_once './core/VariableStream.class.php';

$GLOBALS['amazonXMLServer']	= 'http://www.amazon.com';
$GLOBALS['amazonXMLUrl']	= 'http://xml.amazon.com/onca/xml3';

$GLOBALS['devtoken']		= $GLOBALS['config']['amazontoken'];	// developer token
#$GLOBALS['token']			= 'cpuidle-20';		                    // associates id

$GLOBALS['searchMode']		= 'dvd';

$GLOBALS['dbgenres']		= null;		// make sure genres are read once (via mapGenres in genres.php)

/*

AG: Moved version check into engines.php

// amazonxml requires php5 xpath functions
if (version_compare(phpversion(), '5.0') < 0)
{
    errorpage('PHP version mismatch',
              'At least PHP version 5.0.0 is required to run the Amazon XML module, please check the documentation!');
}

// amazonxml requires valid amazon developer id
if (empty($GLOBALS['devtoken']))
{
	errorpage('Developer Token Missing', 'An <a href="'.$amazonXMLServer.'">Amazon</a> developer token '.
              'is required to run this module, please register for using Amazon web services!');
}
*/

/**
 * Get meta information about the engine
 *
 * @todo    Include image search capabilities etc in meta information
 */
function amazonxmlMeta()
{
    return array('name' => 'Amazon (XML)', 'stable' => 1, 'php' => '5.0');
}

/**
 * Check if valid developer token is present
 */
function amazonxmlCheckToken()
{
    global $devtoken;

    if (empty($devtoken))
    {
        errorpage('Developer Token Missing', 'An <a href="'.$amazonXMLServer.'">Amazon</a> developer token '.
                  'is required to run this module, please register for using Amazon web services!');
    }
}

/**
 * Perform XPath query and return
 * concatenated result list
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   domDocument parent  Parent XML document node to start search
 * @param   string      name    Query string
 * @param   string      delim   Delimiter for concatenation
 * @return  string    	        The concatenated result set
 */
function getElement($parent, $name, $delim = ', ')
{
	global $xpath;

	$nodeList = $xpath->query($name, $parent);
	$result = '';

	for ($i = 0; $i < $nodeList->length; $i++)
    {
        $node = $nodeList->item($i);
		if ($result) $result .= $delim;
		$result .= $node->nodeValue;
	}
	
	return trim($result);
}

/**
 * Convert XML result into videoDB result array
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   domNode dvd     Parent XML document node to start search
 * @param   boolean short   Return title & id only
 * @return  array    	    The converted result set for a single item
 */
function getDataFromNode($dvd, $short = false)
{
	global $xpath;

	// Id
	$data['id']			= getElement($dvd, 'Asin');

	// Title
	$data['title']		= getElement($dvd, 'ProductName');
	$data['subtitle']	= '';

	if ($short) return $data;	// exit if called from search function
	
	// Year
	$date = getElement($dvd, 'TheatricalReleaseDate');
	if (empty($date)) $date = getElement($dvd, 'ReleaseDate');
	if (preg_match('/\d{4}/', $date, $ary)) {
		$data['year']	= $ary[0];
	}
	
	// Cover URL
	$data['coverurl']	= getElement($dvd, 'ImageUrlLarge');

	// MPAA Rating
	$data['mpaa']		= getElement($dvd, 'MpaaRating');

	// Runtime
#	$data['runtime']	= preg_replace('/,/', '', $ary[1]));

	// Director
	$data['director']	= getElement($dvd, 'Directors/Director');

	// Rating
	if ($rating = getElement($dvd, 'Reviews/AvgCustomerRating'))
    {
		$rating = round($rating * 2, 1);
		$data['rating']	= $rating;
	}

	// Countries
#	$data['country']  = join(' ',$ary[1]));

	// Media
	$data['mediatype']  = getElement($dvd, 'Media');

	// Genres (as Array), map to nearest match
	$genres = getElement($dvd, 'BrowseList/BrowseNode/BrowseName', ',');
	foreach (mapGenres($data['genres']) as $genre)
    {
		$data['genres'][] = $genre;
	}

	// Cast
	$data['cast']		= getElement($dvd, 'Starring/Actor', "\n");

	// Plot
	$data['plot']		= getElement($dvd, 'ProductDescription');
	if (empty($plot))
    {
        // use customer reviews if plot not found
		$data['plot']	= getElement($dvd, 'Reviews/CustomerReview/Comment', "\n\n");
	}
    // de-html
    $data['plot']       = preg_replace('#<(br|p)\s*/?>#i', "\n", $data['plot']);
    // avoid double line breaks
    $data['plot']       = preg_replace('/\n+/', "\n", $data['plot']);
	
	// Tracks
	$tracks				= getElement($dvd, 'Tracks/Track', "\n");
	$data['tracks']		= $tracks;

    // convert to numbered track list
	$track_number		= 1;
	foreach (split("\n", $tracks) as $track)
    {
		$track = trim($track);
		if ($tracks) {
			$tracks_formatted .= $track_number++;
			$tracks_formatted .= '. '.$track."\n";
		}
	}

    // append to plot as long as we don't have a separate field
	if ($data['plot']) $data['plot'] .= "\n\n";
	$data['plot']   .= $tracks_formatted;

	return $data;
}

/**
 * Get search Url for an Amazon product
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function amazonxmlSearchUrl($title)
{
	global $amazonXMLServer;
	return $amazonXMLServer;
}

/**
 * Get search Url to visit external site
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string	$id	The movie's external id
 * @return  string		The visit URL
 */
function amazonxmlContentUrl($id)
{
	global $amazonXMLServer;
	return $amazonXMLServer.'/exec/obidos/ASIN/'.$id;
}

/**
 * Search a Movie/DVD/Book etc
 *
 * Searches for a given title on Amazon and returns the found links in
 * an array
 *
 * @author  Andreas Goetz (cpuidle@gmx.de)
 * @param   string    The search string
 * @param   string    The amazon catalog to search in (dvd, books etc)
 * @return  array     Associative array with id and title
 */
function amazonxmlSearch($title, $searchMode = 'dvd', $area = 'us')
{
	global $amazonXMLUrl, $cache, $devtoken;
    #, $token;
	global $CLIENTERROR;
	global $xpath;
	
    // make sure valid token exists
    amazonxmlCheckToken();

	$page		= 1;

	// mapping to ensure matching area, catalog and url
	switch (strtolower($area))
    {
		case 'de':	$area = '-de';
					$amazonXMLUrl = 'http://xml-eu.amazon.com/onca/xml3';
					if ($searchMode == 'music') $searchMode = 'pop-music';	// exception according to AWS guide
					break;
		case 'uk':	$area = '-uk';
					$amazonXMLUrl = 'http://xml-eu.amazon.com/onca/xml3';
					if ($searchMode == 'classical') $area = '';				// exception according to AWS guide
					break;
		case 'jp':	$area = '-jp';
					break;
		default:	$area = '';
	}
	$searchMode .= $area;
	
	$xml = new domDocument();
	$xml->preserveWhiteSpace = false;

	do
	{
		$resp = httpClient($amazonXMLUrl.'?type=heavy&f=xml'.
#                    '&t='.$token.
                    '&t='.AMAZON_TOKEN.
					'&dev-t='.$devtoken.
					'&mode='.$searchMode.
					'&KeywordSearch='.urlencode($title).
					'&page='.$page, $cache);

		if ($resp['success'])
		{
#			echo nl2br(htmlspecialchars(print_r($resp['data'],1)));

			// put data into global scope and load it
			$GLOBALS['xmldata'] = $resp['data'];
			$xml->load('var://xmldata');
			$xpath = new domXPath($xml);
			if (!$total)
            {
                // php4-compatible notation
                $nodes  = $xpath->query('TotalPages');
                $node   = $nodes->item(0);
                $total  = $node->nodeValue;	// total number of pages to load
            }

			$dvds = $xpath->query('Details');
			for ($i = 0; $i < $dvds->length; $i++)
            {
				$dvd	= $dvds->item($i);
				$data	= getDataFromNode($dvd, true);

                // small image for overview
				$data['imgsmall']	= getElement($dvd, 'ImageUrlSmall');

                // cover url for image lookup
                $data['coverurl']   = getElement($dvd, 'ImageUrlLarge');
                if (empty($data['coverurl']))
                {
                    $data['coverurl'] = $data['imgsmall'];
                }

                // description
				$data['details']	= getElement($dvd, 'ProductDescription');
#				echo nl2br(htmlspecialchars(print_r($data,1)));
				$ary[]	= $data;
			}
		}
		else $CLIENTERROR .= $resp['error']."\n";		
	}	
	while (($page++ < $total) && $resp['success']);
	
	return $ary;	
}

/**
 * Fetches the data for a given Amazon ID (equals ISBN)
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   int   IMDB-ID
 * @return  array Result data
 */
function amazonxmlData($id)
{
	global $amazonXMLUrl, $searchMode, $cache, $devtoken;
    #, $token;
	global $CLIENTERROR;
	global $xpath;

    // make sure valid token exists
    amazonxmlCheckToken();

	$xml = new domDocument();
	$xml->preserveWhiteSpace = false;

	$resp = httpClient($amazonXMLUrl.'?type=heavy&f=xml'.
#                '&t='.$token.
                '&t='.AMAZON_TOKEN.
				'&dev-t='.$devtoken.
				'&mode='.$searchMode.
				'&AsinSearch='.urlencode($id).
				'&page=1', $cache);

	if ($resp['success'])
	{
		// put data into global scope and load it
		$GLOBALS['xmldata'] = $resp['data'];
		$xml->load('var://xmldata');
		$xpath = new domXPath($xml);
#		echo nl2br(htmlspecialchars($resp['data']));

		$dvds = $xpath->query('Details');
		$data = getDataFromNode($dvds->item(0));
#		echo nl2br(htmlspecialchars(print_r($data,1)));
	}
	else $CLIENTERROR .= $resp['error']."\n";
	
	return $data;
}

?>