<?php
/**
 * Multi-engine glue logic
 *
 * @package Engines
 * @todo    Add function comments
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @version $Id: engines.php,v 1.19 2006/01/06 10:24:59 andig2 Exp $
 */

require_once './core/httpclient.php';

$GLOBALS['cache'] = true;

/**
 * Determine engine from id
 *
 * @todo    Enhance DB schema to store engine type explicitly
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    item id
 * @return  string    engine name
 */
function engineGetEngine($id)
{
    global $config;
    
	// recognize engine from id
	if ($id)
	{
        // engine prefixed (imdb:081547)
        if (preg_match('/^(\w+):/', $id, $match)) $engine = $match[1];
        elseif (preg_match('/^\d+-\d+$/', $id)) $engine = 'tvtome';
        elseif (preg_match('/^DP[0-9]/', $id)) $engine = 'dvdpalace'; // German Movie Database
        elseif (preg_match('/^[0-9A-Z]{10,}$/', $id))
        {
            if ($config['engine']['amazonxml'])
                $engine = 'amazonxml';
            elseif ($config['engine']['amazoncom'])
                $engine = 'amazoncom';
            else    
                $engine = 'amazon';
        }    
        elseif (preg_match('/^GR[0-9]/', $id)) $engine = 'gamerankings';
        elseif (preg_match('/^DI[0-9]/', $id)) $engine = 'dvdinside';
#		elseif (preg_match('/^[0-9a-z]{6,}$/', $id)) $engine = 'freedb';
	}
	if (empty($engine)) $engine = 'imdb';
	return $engine;
}

/**
 * Include engine file and retrieve item data
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    item id
 * @param   string    engine name
 * @return  array     item data
 */
function engineGetData($id, $engine = 'imdb')
{
	require_once($engine.'.php');
	$func = $engine.'Data';

    $result = array();
    if (function_exists($func))
    {
        $result = $func($id);
    }

	return $result;
}

/**
 * Get item details URL in external site
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    item id
 * @param   string    engine name
 * @return  string    item details url
 */
function engineGetContentUrl($id, $engine = 'imdb')
{
    if (empty($id)) return '';
    
    require_once($engine.'.php');
    $func = $engine.'ContentUrl';
    
    $result = '';
    if (function_exists($func))
    {
        $result = $func($id);
    }

    return $result;
}

/**
 * Get complete search URL for external site
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    search string
 * @param   string    engine name
 * @return  string    item search url
 */
function engineGetSearchUrl($find, $engine = 'imdb')
{
    require_once($engine.'.php');
    $func = $engine.'SearchUrl';
    
    $result = '';
    if (function_exists($func))
    {
        $result = $func($find);
    }

    return $result;
}

/**
 * Include engine file and execute item search
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    search string
 * @param   string    engine name
 * @return  array     list of item data
 */
function engineSearch($find, $engine = 'imdb')
{
    require_once($engine.'.php');
    $func = $engine.'Search';

    $result = array();
    if (function_exists($func))
    {
        $result = $func($find);
    }

    return $result;
}

/**
 * Constructs engines array for use in lookup template
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    search string
 * @param   string    search type (for Amazon)
 * @param   string    engine name
 * @return  array     engines array for lookup template
 */
function engineGetLookup($find, $searchtype, $engine = 'imdb')
{
    global $config, $engines, $lang;

    $tpl = array();

    foreach ($engines as $key => $value)
    {
        if ($value)
        {
            // active indicator
            $tpl[$key]['CLASS'] = ($key == $engine) ? 'tabActive' : 'tabInactive';
            
            // url
            $tpl[$key]['url']   = 'lookup.php?find='.urlencode($find).'&engine='.$key.'&searchtype='.$searchtype;
            
            // title
            $tpl[$key]['name']  = $config['engines'][$key]['name'];
        }
    }

    return $tpl;
}

/**
 * Retrieve meta information about all available engines
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @return  array     engines array containing engine names
 */
function engineMeta()
{
    $engines = array();
    
    if ($dh = @opendir('./engines'))
    {
        while (($file = readdir($dh)) !== false)
        {
            if ((preg_match("/(.*)\.php$/", $file, $matches)) && ($matches[1] != 'engines'))
            {
                // engine file
                $engine           = $matches[1];            
                $engines[$engine] = $engine;

                // get meta data
                require_once('./engines/'.$engine.'.php');
                $func = $engine.'Meta';

                if (function_exists($func))
                {
                    $engines[$engine] = $func();
                    
                    // required php version present?
                    if ($engines[$engine]['php'] && (version_compare(phpversion(), $engines[$engine]['php']) < 0))
                    {
                        unset($engines[$engine]);
                    }    
                }    
            }
        }
        closedir($dh);
    }

    return $engines;
}

?>
