<?php
/**
 * IMDB Parser
 *
 * Parses data from the Internet Movie Database
 *
 * @package Engines
 * @author  Andreas Gohr    <a.gohr@web.de>
 * @link    http://www.imdb.com  Internet Movie Database
 * @version $Id: imdb.php,v 1.25 2006/01/27 10:01:11 andig2 Exp $
 */

$GLOBALS['imdbServer']	 = 'http://uk.imdb.com';
$GLOBALS['imdbIdPrefix'] = 'imdb:';

/**
 * Get meta information about the engine
 *
 * @todo    Include image search capabilities etc in meta information
 */
function imdbMeta()
{
    return array('name' => 'IMDB', 'stable' => 1);
}


/**
 * Get Url to search IMDB for a movie
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function imdbSearchUrl($title)
{
	global $imdbServer;
	return $imdbServer.'/Find?'.urlencode($title);
}

/**
 * Get Url to visit IMDB for a specific movie
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string	$id	The movie's external id
 * @return  string		The visit URL
 */
function imdbContentUrl($id)
{
	global $imdbServer;
	global $imdbIdPrefix;
	$id = preg_replace('/^'.$imdbIdPrefix.'/', '', $id);
	return $imdbServer.'/title/tt'.$id.'/';
}

/**
 * Search a Movie
 *
 * Searches for a given title on the IMDB and returns the found links in
 * an array
 *
 * @author  Tiago Fonseca <t_r_fonseca@yahoo.co.uk>
 * @author  Charles Morgan <cmorgan34@yahoo.com>
 * @param   string    The search string
 * @return  array     Associative array with id and title
 */
function imdbSearch($title)
{
    global $imdbServer;
    global $imdbIdPrefix;
    global $CLIENTERROR;

    $resp = httpClient($imdbServer.'/find?q='.urlencode($title), 1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // direct match (redirecting to individual title)?
    $single = array();
    if (preg_match('/^'.preg_quote($imdbServer,'/').'\/[Tt]itle(\?|\/tt)(\d+)\/?/', $resp['url'], $single))
    {
        $ary[0]['id'] = $imdbIdPrefix.$single[2];

        // Title
        preg_match('/<TITLE>(.*?) \([1-2][0-9][0-9][0-9].*?\)<\/TITLE>/i', $resp['data'], $m);
        list($t, $s)        = split(' - ', trim($m[1]), 2);
        $ary[0]['title']    = trim($t);
        $ary[0]['subtitle'] = trim($s);

        return $ary;
    }

    // multiple matches
    preg_match_all('/<A HREF="\/title(\?|\/tt)(\d+)\/.*?">(.+?)<\/A>(\s+(\(.+?\)))?/i', $resp['data'], $data, PREG_SET_ORDER);
    foreach ($data as $row) 
    {
        $info['id']     = $imdbIdPrefix.$row[2];
        $info['title']  = $row[3];
        // add year (helpful in case of multiple matches)
        if (isset($row[5])) $info['title'] .= ' '.$row[5];
        $ary[]          = $info;
    }

    return $ary;
}

/**
 * Fetches the data for a given IMDB-ID
 *
 * @author  Tiago Fonseca <t_r_fonseca@yahoo.co.uk>
 * @param   int   IMDB-ID
 * @return  array Result data
 */
function imdbData($imdbID) 
{
    global $imdbServer;
    global $imdbIdPrefix;
    global $CLIENTERROR;

    $imdbID = preg_replace('/^'.$imdbIdPrefix.'/', '', $imdbID);
    $data= array();	// result
    $ary = array();	// temp

    // fetch mainpage
    $resp = httpClient($imdbServer.'/title/tt'.$imdbID.'/', 1);		// added trailing / to avoid redirect
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // Titles
    preg_match('/<TITLE>(.*?) \([1-2][0-9][0-9][0-9].*?\)<\/TITLE>/i', $resp['data'], $ary);
    list($t, $s)      = split(' - ', trim($ary[1]), 2);
    $data['title']    = trim($t);
    $data['subtitle'] = trim($s);

    // Year
    preg_match('/<A HREF="\/Sections\/Years\/[1-2][0-9][0-9][0-9]\/?">([1-2][0-9][0-9][0-9])<\/A>/i', $resp['data'], $ary);
    $data['year']     = trim($ary[1]);

    // Cover URL
#    preg_match('/<IMG.*?alt="cover".*?(http:\/\/.+?\.(jpe?g|gif))/i', $resp['data'], $ary);
    preg_match('/name="poster".+?<IMG.+?(http:\/\/.+?\.(jpe?g|gif))/i', $resp['data'], $ary);
    $data['coverurl'] = trim($ary[1]);

    // MPAA Rating
    preg_match('/<A HREF="\/mpaa">MPAA<\/A>: ?<\/B>(.+?)<br>/i', $resp['data'], $ary);
    $data['mpaa']     = trim($ary[1]);

    // UK BBFC Rating
    preg_match('/>UK:(.*?)<\/a>\s+/i', $resp['data'], $ary);
    $data['bbfc'] = trim($ary[1]);
    
    // Runtime
    preg_match('/Runtime:?<\/B>:?.*?([0-9,]+).*?<\/TD>/si', $resp['data'], $ary);
    $data['runtime']  = preg_replace('/,/', '', trim($ary[1]));

    // Director
    preg_match('/Directed by<\/B><BR>\s+?<A HREF="\/Name[?\/].+?">(.+?)<\/A>/i', $resp['data'], $ary);
    $data['director'] = trim($ary[1]);

    // Rating
    preg_match('/<b>(\d{1,2}\.\d)\/10<\/b> \(.*? votes\)/i', $resp['data'], $ary);
    $data['rating']   = trim($ary[1]);

    // Countries
    preg_match_all('/<A HREF="\/Sections\/Countries\/.+?\/">(.+?)<\/A>/i', $resp['data'], $ary, PREG_PATTERN_ORDER);
    $data['country']  = trim(join(', ', $ary[1]));

    // Languages
    preg_match_all('/<A HREF="\/Sections\/Languages\/.+?\/">(.+?)<\/A>/i', $resp['data'], $ary, PREG_PATTERN_ORDER);
    $data['language'] = trim(strtolower(join(', ', $ary[1])));

    // Plot (movies in their early stages have the plot here but not yet in plotsummary?)
    preg_match('/<b.*?>Plot Outline:<\/b>\s+(.*?)<br>/is', $resp['data'], $ary);
    if (!empty($ary[1])) $data['plot'] = trim($ary[1]);

    // Genres (as Array)
    preg_match_all('/<A HREF="\/Sections\/Genres\/.+?\/">(.+?)<\/A>/i', $resp['data'], $ary, PREG_PATTERN_ORDER);
    foreach($ary[1] as $genre)
    {
        $data['genres'][] = trim($genre);
    }

    // fetch credits
    $resp = httpClient($imdbServer.'/title/tt'.$imdbID.'/fullcredits', 1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";
    $resp['data'] = preg_replace('/.*?Cast<\/B><\/A>(.*?)<\/TABLE>.*/is', '$1', $resp['data']);

    // Cast
    preg_match_all('/<A HREF="\/Name[?\/](.*?)">(.*?)<\/A>.*?(\.\.\.\. )?<\/TD><TD VALIGN="TOP">(.*?)<\/TD>/si', $resp['data'], $ary,PREG_PATTERN_ORDER);
    $count = 0;
    while (isset($ary[2][$count])) 
    {
        $character = $ary[4][$count];
        if (preg_match("/^<a href=.*?>(.*?)<\/a>/i", $character, $matches)) $character = $matches[1];
        $actor  = trim(strip_tags($ary[2][$count]));
        $role   = trim(strip_tags($character));
        $actorid= (preg_match("/(.*?)\//", $ary[1][$count], $m)) ? $m[1] : '';
        $cast  .= "$actor::$role::$actorid\n";
        $count++;
    }
    $data['cast'] = trim($cast);

    // fetch Plot
    $resp = httpClient($imdbServer.'/title/tt'.$imdbID.'/plotsummary', 1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // Plot
    preg_match('/<P CLASS="plotpar">(.+?)<\/P>/is', $resp['data'], $ary);
    if (!empty($ary[1])) $data['plot'] = trim($ary[1]);
    $data['plot'] = preg_replace('/[\n\r]/',' ', $data['plot']);
    $data['plot'] = preg_replace('/  /',' ', $data['plot']);
    $data['plot'] = trim($data['plot']);

    return $data;
}

/**
 * Parses Actor-Details
 *
 * Find image and detail URL for actor, not sure if this can be made
 * a one-step process?
 *
 * @author                Andreas Goetz <cpuidle@gmx.de>
 * @param  string  $name  Name of the Actor
 * @return array          array with Actor-URL and Thumbnail
 */
function imdbActor($name, $actorid)
{
    global $imdbServer;
    
    // search directly by id or via name?
    $url    = ($actorid) ? '/name/'.urlencode($actorid).'/' : '/Name?'.urlencode($name);
    $resp   = httpClient($imdbServer.$url, 1);

    $ary    = array();
    
    // if not direct match load best match
    if (preg_match('#<b>Popular Names</b>.+?<a\s+href="(.*?)">#i', $resp['data'], $m) ||
		preg_match('#<b>Names \(Exact Matches\)</b>.+?<a\s+href="(.*?)">#i', $resp['data'], $m) ||
		preg_match('#<b>Names \(Approx Matches\)</b>.+?<a\s+href="(.*?)">#i', $resp['data'], $m) ) {

        if (!preg_match('/http/i', $m[1])) $m[1] = $imdbServer.$m[1];
        $resp = httpClient($m[1], true);
    }

	// now we should have loaded the best match    
    if (preg_match('/<a\s+name="headshot"\s+href="(.+?)">\s*<img\s+.*?src="(.*?)"/i', $resp['data'], $m))
    {
/*    
echo nl2br(htmlentities(print_r($m[0],1)));
echo "<br/><br/>";
echo htmlentities($m[2]);
echo "<br/><br/>";
*/
        $ary[0][0] = $m[1];
        $ary[0][1] = $m[2];
        return $ary;
    } 
    else return null;
}

?>
